package de.ozgcloud.eingang.router;

import java.util.Objects;
import java.util.concurrent.TimeUnit;

import de.ozgcloud.eingang.common.errorhandling.TechnicalException;
import io.grpc.ManagedChannel;
import io.grpc.stub.AbstractStub;
import lombok.Builder;

@Builder
class ClosableStub<T extends AbstractStub<?>> implements ManagableStub<T> {

	private static final int SHUTDOWN_TIME_IN_SEC = 3;

	private T stub;
	private ManagedChannel channel;

	@Override
	public T get() {
		return stub;
	}

	public void close() {
		if (Objects.nonNull(stub)) {
			shutdownChannel();
			stub = null;
		}
	}

	private void shutdownChannel() {
		try {
			channel.shutdown().awaitTermination(ClosableStub.SHUTDOWN_TIME_IN_SEC, TimeUnit.SECONDS);
		} catch (InterruptedException e) {
			Thread.currentThread().interrupt();
			throw new TechnicalException("Error shutting down grpc channel.", e);
		}
	}

	@Override
	public boolean isShutdownable() {
		return true;
	}
}