/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.router;

import java.util.List;
import java.util.Optional;

import org.springframework.stereotype.Service;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormHeader;
import de.ozgcloud.eingang.common.formdata.ServiceKonto;
import de.ozgcloud.eingang.common.formdata.ZustaendigeStelle;
import lombok.AllArgsConstructor;

@AllArgsConstructor
@Service
public class VorgangService {

	private final VorgangRemoteService remoteService;

	public String createVorgang(FormData formData) {
		var preparedFormData = preserveConsistency(formData);

		return remoteService.createVorgang(preparedFormData, getOrganisationsEinheitId(preparedFormData));
	}

	private Optional<String> getOrganisationsEinheitId(FormData formData) {
		return Optional.ofNullable(formData.getZustaendigeStelle()).map(ZustaendigeStelle::getOrganisationseinheitenId);
	}

	FormData preserveConsistency(FormData formData) {
		if (hasNoPostfachAddress(formData)) {
			return clearServiceKonto(formData);
		}
		return formData;
	}

	private boolean hasNoPostfachAddress(FormData formData) {
		return Optional.ofNullable(formData.getHeader()).map(FormHeader::getServiceKonto).map(ServiceKonto::getPostfachAddresses).map(List::isEmpty)
				.orElse(true);
	}

	private FormData clearServiceKonto(FormData formData) {
		return formData.toBuilder().header(formData.getHeader().toBuilder().serviceKonto(null).build()).build();
	}
}